@@ -11,7 +11,7 @@ from ase.atom import Atom
 from ase.atoms import Atoms
 
 __all__ = ['Atoms', 'Atom']
-__version__ = '3.16.2'
+__version__ = '3.17.0'
 
 # import ase.parallel early to avoid circular import problems when
 # ase.parallel does "from gpaw.mpi import world":
@@ -19,7 +19,5 @@ import ase.parallel  # noqa
 ase.parallel  # silence pyflakes
 
 if LooseVersion(np.__version__) < '1.9':
-    # Make isinstance(x, numbers.Integral) work also for np.intxx:
-    import numbers
-    numbers.Integral.register(np.integer)
-    del numbers
+    raise ImportError(
+        'ASE needs NumPy-1.9.0 or later. You have:', np.version)
