@@ -307,8 +307,8 @@ by invoking the get_potential_energy() method::
 
         self.lmp.command(cell_cmd)
 
-    def set_lammps_pos(self, atoms):
-        pos = atoms.get_positions() / unit_convert("distance", self.units)
+    def set_lammps_pos(self, atoms, wrap=True):
+        pos = atoms.get_positions(wrap=wrap) / unit_convert("distance", self.units)
 
         # If necessary, transform the positions to new coordinate system
         if self.coord_transform is not None:
@@ -618,6 +618,8 @@ by invoking the get_potential_energy() method::
             self.lmp.command('echo none')  # don't echo the atom positions
             self.rebuild(atoms)
             self.lmp.command('echo log')  # turn back on
+        else:
+            self.previous_atoms_numbers = atoms.numbers.copy()
 
         # execute the user commands
         for cmd in self.parameters.lmpcmds:
