@@ -283,7 +283,7 @@ class Siesta3_2(BaseSiesta):
         ('MD.TargetStress', None),
         ('MD.RemoveIntramolecularPressure', False),
         ('MD.InitialTimeStep', 1),
-        ('MD.FinalTimestep', 1),
+        ('MD.FinalTimeStep', 1),
         ('MD.LengthTimeStep', 1.0),  # fs
         ('MD.InitialTemperature', 0.0),  # K
         ('MD.TargetTemperature', 0.0),  # K
@@ -292,6 +292,11 @@ class Siesta3_2(BaseSiesta):
         ('MD.AnnealOption', 0.0),
         ('MD.TauRelax', 100.0),  # fs
         ('MD.BulkModulus', 100.0),  # Ry/Bohr^{3}
+        ('Master.code', 'fsiesta'),
+        ('Master.interface', 'pipes'),
+        ('Master.address', 'localhost'),  # or IP address
+        ('Master.port', 10001),
+        ('Master.socketType', 'inet'),
         ('WriteCoorInitial', True),
         ('WriteCoorStep', False),
         ('WriteForces', False),
@@ -573,7 +578,7 @@ class SiestaTrunk462(BaseSiesta):
         ('MD.TargetStress', None),
         ('MD.RemoveIntramolecularPressure', False),
         ('MD.InitialTimeStep', 1),
-        ('MD.FinalTimestep', 1),
+        ('MD.FinalTimeStep', 1),
         ('MD.LengthTimeStep', 1.0),  # fs
         ('MD.InitialTemperature', 0.0),  # K
         ('MD.TargetTemperature', 0.0),  # K
