@@ -22,10 +22,10 @@ import sys
 
 import numpy as np
 
+import ase.io.jsonio
 from ase.data import atomic_numbers
 from ase.db.row import AtomsRow
 from ase.db.core import Database, ops, now, lock, invop, parse_selection
-from ase.io.jsonio import encode, decode
 from ase.parallel import parallel_function
 from ase.utils import basestring
 
@@ -120,6 +120,7 @@ def float_if_not_none(x):
 
 
 class SQLite3Database(Database, object):
+    type = 'db'
     initialized = False
     _allow_reading_old_format = False
     default = 'NULL'  # used for autoincrement id
@@ -128,6 +129,41 @@ class SQLite3Database(Database, object):
     columnnames = [line.split()[0].lstrip()
                    for line in init_statements[0].splitlines()[1:]]
 
+    def encode(self, obj):
+        return ase.io.jsonio.encode(obj)
+
+    def decode(self, txt):
+        return ase.io.jsonio.decode(txt)
+
+    def blob(self, array):
+        """Convert array to blob/buffer object."""
+
+        if array is None:
+            return None
+        if len(array) == 0:
+            array = np.zeros(0)
+        if array.dtype == np.int64:
+            array = array.astype(np.int32)
+        if not np.little_endian:
+            array = array.byteswap()
+        return buffer(np.ascontiguousarray(array))
+
+    def deblob(self, buf, dtype=float, shape=None):
+        """Convert blob/buffer object to ndarray of correct dtype and shape.
+
+        (without creating an extra view)."""
+        if buf is None:
+            return None
+        if len(buf) == 0:
+            array = np.zeros(0, dtype)
+        else:
+            array = np.frombuffer(buf, dtype)
+            if not np.little_endian:
+                array = array.byteswap()
+        if shape is not None:
+            array.shape = shape
+        return array
+
     def _connect(self):
         return sqlite3.connect(self.filename, timeout=600)
 
@@ -192,29 +228,34 @@ class SQLite3Database(Database, object):
 
         self.initialized = True
 
-    def _write(self, atoms, key_value_pairs, data):
+    def _write(self, atoms, key_value_pairs, data, id):
         Database._write(self, atoms, key_value_pairs, data)
+        encode = self.encode
 
         con = self.connection or self._connect()
         self._initialize(con)
         cur = con.cursor()
 
-        id = None
+        mtime = now()
+
+        blob = self.blob
+
+        text_key_values = []
+        number_key_values = []
 
         if not isinstance(atoms, AtomsRow):
             row = AtomsRow(atoms)
-            row.ctime = mtime = now()
+            row.ctime = mtime
             row.user = os.getenv('USER')
         else:
             row = atoms
-            cur.execute('SELECT id FROM systems WHERE unique_id=?',
-                        (row.unique_id,))
-            results = cur.fetchall()
-            if results:
-                id = results[0][0]
-                self._delete(cur, [id], ['keys', 'text_key_values',
-                                         'number_key_values'])
-            mtime = now()
+
+        if id:
+            self._delete(cur, [id], ['keys', 'text_key_values',
+                                     'number_key_values', 'species'])
+        else:
+            if not key_value_pairs:
+                key_value_pairs = row.key_value_pairs
 
         constraints = row._constraints
         if constraints:
@@ -243,9 +284,6 @@ class SQLite3Database(Database, object):
         else:
             values += (None, None)
 
-        if key_value_pairs is None:
-            key_value_pairs = row.key_value_pairs
-
         if not data:
             data = row._data
         if not isinstance(data, basestring):
@@ -272,20 +310,18 @@ class SQLite3Database(Database, object):
             q = self.default + ', ' + ', '.join('?' * len(values))
             cur.execute('INSERT INTO systems VALUES ({})'.format(q),
                         values)
+            id = self.get_last_id(cur)
         else:
             q = ', '.join(name + '=?' for name in self.columnnames[1:])
             cur.execute('UPDATE systems SET {} WHERE id=?'.format(q),
                         values + (id,))
 
-        if id is None:
-            id = self.get_last_id(cur)
-
-            count = row.count_atoms()
-            if count:
-                species = [(atomic_numbers[symbol], n, id)
-                           for symbol, n in count.items()]
-                cur.executemany('INSERT INTO species VALUES (?, ?, ?)',
-                                species)
+        count = row.count_atoms()
+        if count:
+            species = [(atomic_numbers[symbol], n, id)
+                       for symbol, n in count.items()]
+            cur.executemany('INSERT INTO species VALUES (?, ?, ?)',
+                            species)
 
         text_key_values = []
         number_key_values = []
@@ -311,8 +347,12 @@ class SQLite3Database(Database, object):
 
     def get_last_id(self, cur):
         cur.execute('SELECT seq FROM sqlite_sequence WHERE name="systems"')
-        id = cur.fetchone()[0]
-        return id
+        result = cur.fetchone()
+        if result is not None:
+            id = result[0]
+            return id
+        else:
+            return 0
 
     def _get_row(self, id):
         con = self._connect()
@@ -330,6 +370,9 @@ class SQLite3Database(Database, object):
         return self._convert_tuple_to_row(values)
 
     def _convert_tuple_to_row(self, values):
+        deblob = self.deblob
+        decode = self.decode
+
         values = self._old2new(values)
         dct = {'id': values[0],
                'unique_id': values[1],
@@ -338,8 +381,10 @@ class SQLite3Database(Database, object):
                'user': values[4],
                'numbers': deblob(values[5], np.int32),
                'positions': deblob(values[6], shape=(-1, 3)),
-               'cell': deblob(values[7], shape=(3, 3)),
-               'pbc': (values[8] & np.array([1, 2, 4])).astype(bool)}
+               'cell': deblob(values[7], shape=(3, 3))}
+
+        if values[8] is not None:
+            dct['pbc'] = (values[8] & np.array([1, 2, 4])).astype(bool)
         if values[9] is not None:
             dct['initial_magmoms'] = deblob(values[9])
         if values[10] is not None:
@@ -354,6 +399,7 @@ class SQLite3Database(Database, object):
             dct['constraints'] = values[14]
         if values[15] is not None:
             dct['calculator'] = values[15]
+        if values[16] is not None:
             dct['calculator_parameters'] = decode(values[16])
         if values[17] is not None:
             dct['energy'] = values[17]
@@ -374,18 +420,20 @@ class SQLite3Database(Database, object):
         if values[25] != '{}':
             dct['key_value_pairs'] = decode(values[25])
         if len(values) >= 27 and values[26] != 'null':
-            dct['data'] = values[26]
+            dct['data'] = decode(values[26])
 
         return AtomsRow(dct)
 
     def _old2new(self, values):
+        if self.type == 'postgresql':
+            assert self.version >= 8, 'Your db-version is too old!'
         assert self.version >= 4, 'Your db-file is too old!'
         if self.version < 5:
             pass  # should be ok for reading by convert.py script
         if self.version < 6:
             m = values[23]
             if m is not None and not isinstance(m, float):
-                magmom = float(deblob(m, shape=()))
+                magmom = float(self.deblob(m, shape=()))
                 values = values[:23] + (magmom,) + values[24:]
         return values
 
@@ -395,19 +443,21 @@ class SQLite3Database(Database, object):
         tables = ['systems']
         where = []
         args = []
-
-        for n, key in enumerate(keys):
+        for key in keys:
             if key == 'forces':
                 where.append('systems.fmax IS NOT NULL')
             elif key == 'strain':
                 where.append('systems.smax IS NOT NULL')
             elif key in ['energy', 'fmax', 'smax',
                          'constraints', 'calculator']:
-                where.append('systems.{0} IS NOT NULL'.format(key))
+                where.append('systems.{} IS NOT NULL'.format(key))
             else:
-                tables.append('keys AS keys{0}'.format(n))
-                where.append('systems.id=keys{0}.id AND keys{0}.key=?'
-                             .format(n))
+                if '-' not in key:
+                    q = 'systems.id in (select id from keys where key=?)'
+                else:
+                    key = key.replace('-', '')
+                    q = 'systems.id not in (select id from keys where key=?)'
+                where.append(q)
                 args.append(key)
 
         # Special handling of "H=0" and "H<2" type of selections:
@@ -416,10 +466,6 @@ class SQLite3Database(Database, object):
             if isinstance(key, int):
                 bad[key] = bad.get(key, True) and ops[op](0, value)
 
-        found_sort_table = False
-        nspecies = 0
-        ntext = 0
-        nnumber = 0
         for key, op, value in cmps:
             if key in ['id', 'energy', 'magmom', 'ctime', 'user',
                        'calculator', 'natoms', 'pbc', 'unique_id',
@@ -430,70 +476,84 @@ class SQLite3Database(Database, object):
                     assert op in ['=', '!=']
                     value = int(np.dot([x == 'T' for x in value], [1, 2, 4]))
                 elif key == 'magmom':
-                    assert self.version >= 6, 'Update you db-file'
-                where.append('systems.{0}{1}?'.format(key, op))
+                    assert self.version >= 6, 'Update your db-file'
+                where.append('systems.{}{}?'.format(key, op))
                 args.append(value)
             elif isinstance(key, int):
-                if bad[key]:
+                if self.type == 'postgresql':
                     where.append(
-                        'NOT EXISTS (SELECT id FROM species WHERE\n' +
-                        '  species.id=systems.id AND species.Z=? AND ' +
-                        'species.n{0}?)'.format(invop[op]))
+                        'cardinality(array_positions(' +
+                        'numbers::int[], ?)){}?'.format(op))
                     args += [key, value]
                 else:
-                    tables.append('species AS specie{0}'.format(nspecies))
-                    where.append(('systems.id=specie{0}.id AND ' +
-                                  'specie{0}.Z=? AND ' +
-                                  'specie{0}.n{1}?').format(nspecies, op))
-                    args += [key, value]
-                    nspecies += 1
+                    if bad[key]:
+                        where.append(
+                            'systems.id not in (select id from species ' +
+                            'where Z=? and n{}?)'.format(invop[op]))
+                        args += [key, value]
+                    else:
+                        where.append('systems.id in (select id from species ' +
+                                     'where Z=? and n{}?)'.format(op))
+                        args += [key, value]
+
+            elif self.type == 'postgresql':
+                jsonop = '->'
+                if isinstance(value, basestring):
+                    jsonop = '->>'
+                elif isinstance(value, bool):
+                    jsonop = '->>'
+                    value = str(value).lower()
+                where.append("systems.key_value_pairs {} '{}'{}?"
+                             .format(jsonop, key, op))
+                args.append(str(value))
+
             elif isinstance(value, basestring):
-                tables.append('text_key_values AS text{0}'.format(ntext))
-                where.append(('systems.id=text{0}.id AND ' +
-                              'text{0}.key=? AND ' +
-                              'text{0}.value{1}?').format(ntext, op))
+                where.append('systems.id in (select id from text_key_values ' +
+                             'where key=? and value{}?)'.format(op))
                 args += [key, value]
-                if sort_table == 'text_key_values' and sort == key:
-                    sort_table = 'text{0}'.format(ntext)
-                    found_sort_table = True
-                ntext += 1
             else:
-                tables.append('number_key_values AS number{0}'.format(nnumber))
-                where.append(('systems.id=number{0}.id AND ' +
-                              'number{0}.key=? AND ' +
-                              'number{0}.value{1}?').format(nnumber, op))
+                where.append(
+                    'systems.id in (select id from number_key_values ' +
+                    'where key=? and value{}?)'.format(op))
                 args += [key, float(value)]
-                if sort_table == 'number_key_values' and sort == key:
-                    sort_table = 'number{0}'.format(nnumber)
-                    found_sort_table = True
-                nnumber += 1
 
         if sort:
-            if sort_table == 'systems':
-                if sort in ['energy', 'fmax', 'smax', 'calculator']:
-                    where.append('systems.{0} IS NOT NULL'.format(sort))
-            else:
-                if not found_sort_table:
-                    tables.append('{0} AS sort_table'.format(sort_table))
-                    where.append('systems.id=sort_table.id AND '
-                                 'sort_table.key=?')
-                    args.append(sort)
-                    sort_table = 'sort_table'
+            if sort_table != 'systems':
+                tables.append('{} AS sort_table'.format(sort_table))
+                where.append('systems.id=sort_table.id AND '
+                             'sort_table.key=?')
+                args.append(sort)
+                sort_table = 'sort_table'
                 sort = 'value'
 
-        sql = 'SELECT {0} FROM\n  '.format(what) + ', '.join(tables)
+        sql = 'SELECT {} FROM\n  '.format(what) + ', '.join(tables)
         if where:
             sql += '\n  WHERE\n  ' + ' AND\n  '.join(where)
         if sort:
-            sql += '\nORDER BY {0}.{1} {2}'.format(sort_table, sort, order)
+            # XXX use "?" instead of "{}"
+            sql += '\nORDER BY {0}.{1} IS NULL, {0}.{1} {2}'.format(
+                sort_table, sort, order)
 
         return sql, args
 
     def _select(self, keys, cmps, explain=False, verbosity=0,
-                limit=None, offset=0, sort=None, include_data=True):
+                limit=None, offset=0, sort=None, include_data=True,
+                columns='all'):
         con = self._connect()
         self._initialize(con)
 
+        values = np.array([None for i in range(27)])
+        values[25] = '{}'
+        values[26] = 'null'
+
+        if columns == 'all':
+            columnindex = list(range(26))
+        else:
+            columnindex = [c for c in range(0, 26)
+                           if self.columnnames[c] in columns]
+        if include_data:
+            columnindex.append(26)
+
         if sort:
             if sort[0] == '-':
                 order = 'DESC'
@@ -505,27 +565,28 @@ class SQLite3Database(Database, object):
                         'fmax', 'smax', 'volume', 'mass', 'charge', 'natoms']:
                 sort_table = 'systems'
             else:
-                for dct in self._select(keys + [sort], cmps, limit=1,
-                                        include_data=False):
+                for dct in self._select(keys + [sort], cmps=[], limit=1,
+                                        include_data=False,
+                                        columns=['key_value_pairs']):
                     if isinstance(dct['key_value_pairs'][sort], basestring):
                         sort_table = 'text_key_values'
                     else:
                         sort_table = 'number_key_values'
                     break
                 else:
-                    return
+                    # No rows.  Just pick a table:
+                    sort_table = 'number_key_values'
+
         else:
             order = None
             sort_table = None
 
-        if include_data:
-            what = 'systems.*'
-        else:
-            what = ', '.join('systems.' + name
-                             for name in self.columnnames[:26])
+        what = ', '.join('systems.' + name
+                         for name in
+                         np.array(self.columnnames)[np.array(columnindex)])
 
-        sql, args = self.create_select_statement(keys, cmps,
-                                                 sort, order, sort_table, what)
+        sql, args = self.create_select_statement(keys, cmps, sort, order,
+                                                 sort_table, what)
 
         if explain:
             sql = 'EXPLAIN QUERY PLAN ' + sql
@@ -545,8 +606,23 @@ class SQLite3Database(Database, object):
             for row in cur.fetchall():
                 yield {'explain': row}
         else:
-            for values in cur.fetchall():
-                yield self._convert_tuple_to_row(values)
+            n = 0
+            for shortvalues in cur.fetchall():
+                values[columnindex] = shortvalues
+                yield self._convert_tuple_to_row(tuple(values))
+                n += 1
+
+            if sort and sort_table != 'systems':
+                # Yield rows without sort key last:
+                if limit is not None:
+                    if n == limit:
+                        return
+                    limit -= n
+                for row in self._select(keys + ['-' + sort], cmps,
+                                        limit=limit, offset=offset,
+                                        include_data=include_data,
+                                        columns=columns):
+                    yield row
 
     @parallel_function
     def count(self, selection=None, **kwargs):
@@ -563,46 +639,11 @@ class SQLite3Database(Database, object):
         self._initialize(con)
         con.execute('ANALYZE')
 
-    def _update(self, ids, delete_keys, add_key_value_pairs):
-        """Update row(s).
-
-        ids: int or list of int
-            ID's of rows to update.
-        delete_keys: list of str
-            Keys to remove.
-        add_key_value_pairs: dict
-            Key-value pairs to add.
-
-        Returns number of key-value pairs added and keys removed.
-        """
-
-        rows = [self._get_row(id) for id in ids]
-        if self.connection:
-            # We are already running inside a context manager:
-            return self._update_rows(rows, delete_keys, add_key_value_pairs)
-
-        # Create new context manager:
-        with self:
-            return self._update_rows(rows, delete_keys, add_key_value_pairs)
-
-    def _update_rows(self, rows, delete_keys, add_key_value_pairs):
-        m = 0
-        n = 0
-        for row in rows:
-            kvp = row.key_value_pairs
-            n += len(kvp)
-            for key in delete_keys:
-                kvp.pop(key, None)
-            n -= len(kvp)
-            m -= len(kvp)
-            kvp.update(add_key_value_pairs)
-            m += len(kvp)
-            self._write(row, kvp, None)
-        return m, n
-
     @parallel_function
     @lock
     def delete(self, ids):
+        if len(ids) == 0:
+            return
         con = self._connect()
         self._delete(con.cursor(), ids)
         con.commit()
@@ -611,8 +652,8 @@ class SQLite3Database(Database, object):
     def _delete(self, cur, ids, tables=None):
         tables = tables or all_tables[::-1]
         for table in tables:
-            cur.executemany('DELETE FROM {0} WHERE id=?'.format(table),
-                            ((id,) for id in ids))
+            cur.execute('DELETE FROM {} WHERE id in ({});'.
+                        format(table, ', '.join([str(id) for id in ids])))
 
     @property
     def metadata(self):
@@ -639,42 +680,6 @@ class SQLite3Database(Database, object):
         con.commit()
 
 
-def blob(array):
-    """Convert array to blob/buffer object."""
-
-    if array is None:
-        return None
-    if len(array) == 0:
-        array = np.zeros(0)
-    if array.dtype == np.int64:
-        array = array.astype(np.int32)
-    if not np.little_endian:
-        array = array.byteswap()
-    return buffer(np.ascontiguousarray(array))
-
-
-def deblob(buf, dtype=float, shape=None):
-    """Convert blob/buffer object to ndarray of correct dtype and shape.
-
-    (without creating an extra view)."""
-
-    if buf is None:
-        return None
-    if len(buf) == 0:
-        array = np.zeros(0, dtype)
-    else:
-        if len(buf) % 2 == 1:
-            # old psycopg2:
-            array = np.fromstring(str(buf)[1:].decode('hex'), dtype)
-        else:
-            array = np.frombuffer(buf, dtype)
-        if not np.little_endian:
-            array = array.byteswap()
-    if shape is not None:
-        array.shape = shape
-    return array
-
-
 if __name__ == '__main__':
     import sys
     from ase.db import connect
