@@ -1,11 +1,14 @@
 from __future__ import print_function
 
+import warnings
 from unittest.case import SkipTest
 
 from ase import Atoms
 
 try:
-    from IPython.display import HTML
+    with warnings.catch_warnings():
+        warnings.simplefilter('ignore', DeprecationWarning)
+        from IPython.display import HTML
     from ase.visualize import x3d
 except ImportError:
     raise SkipTest('cannot import HTML from IPython.displacy')
@@ -21,4 +24,4 @@ else:
                     cell=(a, a, a),
                     pbc=True)
     my_obj = x3d.view_x3d(atoms)
-    assert isinstance(my_obj, HTML)
\ No newline at end of file
+    assert isinstance(my_obj, HTML)
