@@ -29,7 +29,7 @@ class Infrared(Vibrations):
 
     The calculator object (calc) linked to the Atoms object (atoms) must
     have the attribute:
-    
+
     >>> calc.get_dipole_moment(atoms)
 
     In addition to the methods included in the ``Vibrations`` class
@@ -66,7 +66,7 @@ class Infrared(Vibrations):
         use the dipole moment in all directions.
 
     Example:
-    
+
     >>> from ase.io import read
     >>> from ase.calculators.vasp import Vasp
     >>> from ase.vibrations import Infrared
@@ -140,19 +140,11 @@ class Infrared(Vibrations):
     """
     def __init__(self, atoms, indices=None, name='ir', delta=0.01,
                  nfree=2, directions=None):
-        assert nfree in [2, 4]
-        self.atoms = atoms
+        Vibrations.__init__(self, atoms, indices=indices, name=name,
+                            delta=delta, nfree=nfree)
         if atoms.constraints:
             print('WARNING! \n Your Atoms object is constrained. '
                   'Some forces may be unintended set to zero. \n')
-        self.calc = atoms.get_calculator()
-        if indices is None:
-            indices = range(len(atoms))
-        self.indices = np.asarray(indices)
-        self.nfree = nfree
-        self.name = name + '-d%.3f' % delta
-        self.delta = delta
-        self.H = None
         if directions is None:
             self.directions = np.asarray([0, 1, 2])
         else:
@@ -332,5 +324,5 @@ class Infrared(Vibrations):
         fd.close()
         # np.savetxt(out, outdata, fmt='%.3f  %15.5e  %15.5e')
 
-        
+
 InfraRed = Infrared  # old name
